(** OCaml bindings for the GLX library. *)
(* Copyright (C) 2008, 2009 by Florent Monnier <fmonnier@linux-nantes.org>
 *
 * OCaml-Xlib is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OCaml-Xlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OCaml-Xlib.  If not, see:
 * <http://www.gnu.org/licenses/>
 *)

(**
  {b Warning:} This module is currently under construction.
               You will not be able to use it yet.

  This module is a binding to this GLX extension:
  {{:http://www.opengl.org/registry/specs/EXT/texture_from_pixmap.txt}GLX_EXT_texture_from_pixmap}
*)

type buffer_param =
  | GLX_FRONT_LEFT_EXT
  | GLX_FRONT_RIGHT_EXT
  | GLX_BACK_LEFT_EXT
  | GLX_BACK_RIGHT_EXT
  | GLX_FRONT_EXT
  | GLX_BACK_EXT
  | GLX_AUX0_EXT
  | GLX_AUX1_EXT
  | GLX_AUX2_EXT
  | GLX_AUX3_EXT
  | GLX_AUX4_EXT
  | GLX_AUX5_EXT
  | GLX_AUX6_EXT
  | GLX_AUX7_EXT
  | GLX_AUX8_EXT
  | GLX_AUX9_EXT

external glXReleaseTexImageEXT: dpy:Xlib.display -> drawable:GLX.glXDrawable -> buffer:buffer_param -> unit
    = "ml_glXReleaseTexImageEXT"

external glXBindTexImageEXT: dpy:Xlib.display -> drawable:GLX.glXDrawable -> buffer:buffer_param ->
    attrib_list:'a list (* TODO *) -> unit
    = "ml_glXBindTexImageEXT"


(* ========================================================================== *(
(* Example taken from:
   http://people.freedesktop.org/~davidr/GLX_EXT_texture_from_pixmap.txt
*)
(* Example 1: Bind redirected window to texture *)
open Xlib
open GLX
open GLX_T2P

let top = ref 0.0
let bottom = ref 1.0


    let attrib = xGetWindowAttributes display window in

    let visualid = xVisualIDFromVisual (attrib.visual);

    let fbconfigs = glXGetFBConfigs display screen in
    (try ArrayLabels.iter nfbconfigs ~f:
      (fun fbconfig ->
        visinfo = glXGetVisualFromFBConfig (display, fbconfig);
        if (!visinfo || visinfo->visualid != visualid)
            continue;

        glXGetFBConfigAttrib (display, fbconfig, GLX_DRAWABLE_TYPE, &value);
        if (!(value & GLX_PIXMAP_BIT))
            continue;

        glXGetFBConfigAttrib (display, fbconfig,
                              GLX_BIND_TO_TEXTURE_TARGETS_EXT,
                              &value);
        if (!(value & GLX_TEXTURE_2D_BIT_EXT))
            continue;

        if (glXGetFBConfigAttrib display fbconfig
                                 GLX_BIND_TO_TEXTURE_RGBA_EXT) = false
        && (glXGetFBConfigAttrib display fbconfig
                                 GLX_BIND_TO_TEXTURE_RGB_EXT) = false
        then () (* continue *) else

        if (glXGetFBConfigAttrib display fbconfig
                                 GLX_Y_INVERTED_EXT)
        then ( top := 0.0; bottom := 1.0; )
        else ( top := 1.0; bottom := 0.0; );

        raise Exit
      )
    with Exit -> ());

    let pixmap = xCompositeNameWindowPixmap display window in
    pixmapAttribs = { GLX_TEXTURE_TARGET_EXT, GLX_TEXTURE_2D_EXT,
                      GLX_TEXTURE_FORMAT_EXT, GLX_TEXTURE_FORMAT_RGBA_EXT,
		      None };
    glxpixmap = glXCreatePixmap (display, fbconfigs[i], pixmap, pixmapAttribs);

    let texture = glGenTexture() in
    glBindTexture GL_TEXTURE_2D texture;

    glXBindTexImageEXT display glxpixmap GLX_FRONT_LEFT_EXT [];

    glTexParameter GL_TEXTURE_2D (TexParam.GL_TEXTURE_MIN_FILTER  Min.GL_LINEAR);
    glTexParameter GL_TEXTURE_2D (TexParam.GL_TEXTURE_MAG_FILTER  Mag.GL_LINEAR);

    (* draw using pixmap as texture *)
    glBegin GL_QUADS;

    glTexCoord2d 0.0 !bottom;
    glVertex2d 0.0 0.0;

    glTexCoord2d 0.0 !top;
    glVertex2d 0.0 1.0;

    glTexCoord2d 1.0 !top;
    glVertex2d 1.0 1.0;

    glTexCoord2d 1.0 !bottom;
    glVertex2d 1.0 0.0;

    glEnd();

    glXReleaseTexImageEXT display glxpixmap GLX_FRONT_LEFT_EXT;

)* *)
