/* OCaml bindings for the GLX library (as part of OCaml-Xlib).
 * Copyright (C) 2008, 2009 by Florent Monnier <fmonnier@linux-nantes.org>
 *
 * OCaml-Xlib is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OCaml-Xlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OCaml-Xlib.  If not, see:
 * <http://www.gnu.org/licenses/>
 */

#include <GL/glx.h>

/* {{{ Begin of GLX_EXT_texture_from_pixmap */

/*
 * For a reason that I have not discoverd yet,
 * this part of <GL/glx.h> gets striped by cpp.
 * These symbols seems to be in my libGL.so.
 * So for now I copy-past it.
 */

#define GLX_EXT_texture_from_pixmap 1

#define GLX_BIND_TO_TEXTURE_RGB_EXT        0x20D0
#define GLX_BIND_TO_TEXTURE_RGBA_EXT       0x20D1
#define GLX_BIND_TO_MIPMAP_TEXTURE_EXT     0x20D2
#define GLX_BIND_TO_TEXTURE_TARGETS_EXT    0x20D3
#define GLX_Y_INVERTED_EXT                 0x20D4

#define GLX_TEXTURE_FORMAT_EXT             0x20D5
#define GLX_TEXTURE_TARGET_EXT             0x20D6
#define GLX_MIPMAP_TEXTURE_EXT             0x20D7

#define GLX_TEXTURE_FORMAT_NONE_EXT        0x20D8
#define GLX_TEXTURE_FORMAT_RGB_EXT         0x20D9
#define GLX_TEXTURE_FORMAT_RGBA_EXT        0x20DA

#define GLX_TEXTURE_1D_BIT_EXT             0x00000001
#define GLX_TEXTURE_2D_BIT_EXT             0x00000002
#define GLX_TEXTURE_RECTANGLE_BIT_EXT      0x00000004

#define GLX_TEXTURE_1D_EXT                 0x20DB
#define GLX_TEXTURE_2D_EXT                 0x20DC
#define GLX_TEXTURE_RECTANGLE_EXT          0x20DD

#define GLX_FRONT_LEFT_EXT                 0x20DE
#define GLX_FRONT_RIGHT_EXT                0x20DF
#define GLX_BACK_LEFT_EXT                  0x20E0
#define GLX_BACK_RIGHT_EXT                 0x20E1
#define GLX_FRONT_EXT                      GLX_FRONT_LEFT_EXT
#define GLX_BACK_EXT                       GLX_BACK_LEFT_EXT
#define GLX_AUX0_EXT                       0x20E2
#define GLX_AUX1_EXT                       0x20E3 
#define GLX_AUX2_EXT                       0x20E4 
#define GLX_AUX3_EXT                       0x20E5 
#define GLX_AUX4_EXT                       0x20E6 
#define GLX_AUX5_EXT                       0x20E7 
#define GLX_AUX6_EXT                       0x20E8
#define GLX_AUX7_EXT                       0x20E9 
#define GLX_AUX8_EXT                       0x20EA 
#define GLX_AUX9_EXT                       0x20EB

void glXBindTexImageEXT(Display *dpy, GLXDrawable drawable, int buffer, const int *attrib_list);
void glXReleaseTexImageEXT(Display *dpy, GLXDrawable drawable, int buffer);

/* }}} End of GLX_EXT_texture_from_pixmap */

#define CAML_NAME_SPACE 1

#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/custom.h>
#include <caml/fail.h>

#include "wrap_xlib.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "wrap_glx.h"


static const int tex_image_buffer_table[] = {
    GLX_FRONT_LEFT_EXT,
    GLX_FRONT_RIGHT_EXT,
    GLX_BACK_LEFT_EXT,
    GLX_BACK_RIGHT_EXT,
    GLX_FRONT_EXT,
    GLX_BACK_EXT,
    GLX_AUX0_EXT,
    GLX_AUX1_EXT,
    GLX_AUX2_EXT,
    GLX_AUX3_EXT,
    GLX_AUX4_EXT,
    GLX_AUX5_EXT,
    GLX_AUX6_EXT,
    GLX_AUX7_EXT,
    GLX_AUX8_EXT,
    GLX_AUX9_EXT
};

CAMLprim value
ml_glXReleaseTexImageEXT( value dpy, value drawable, value buffer )
{
    glXReleaseTexImageEXT(
        Display_val(dpy),
        GLXDrawable_val(drawable),
        tex_image_buffer_table[Long_val(buffer)]
    );
    return Val_unit;
}

CAMLprim value
ml_glXBindTexImageEXT( value dpy, value drawable, value buffer, value ml_attrib_list )
{
    glXBindTexImageEXT(
        Display_val(dpy),
        GLXDrawable_val(drawable),
        tex_image_buffer_table[Long_val(buffer)],
        NULL // const int *attrib_list  /* TODO */
    );
    return Val_unit;
}


// vim: sw=4 sts=4 ts=4 et fdm=marker
